import { Directive, ElementRef, Injectable, InjectionToken, Input, NgModule, Renderer2 } from '@angular/core';
import { Subject } from 'rxjs/Subject';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/observable/from';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/take';
import 'rxjs/add/operator/delay';
import 'rxjs/add/operator/skipWhile';
var HighlightService = (function () {
    /**
     * @param {?=} theme
     * @param {?=} path
     */
    function HighlightService(theme, path) {
        if (theme === void 0) { theme = 'github'; }
        if (path === void 0) { path = 'assets/lib/hljs'; }
        this.theme = theme;
        this.path = path;
    }
    return HighlightService;
}());
HighlightService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
HighlightService.ctorParameters = function () { return [
    null,
    null,
]; };
var HighlightDirective = (function () {
    /**
     * @param {?} el
     * @param {?} renderer
     * @param {?} hl
     */
    function HighlightDirective(el, renderer, hl) {
        this.renderer = renderer;
        this.hl = hl;
        this.highlighter$ = new Subject();
        this.hlAuto = true;
        this.hlDelay = 200;
        this.el = el.nativeElement;
    }
    /**
     * @return {?}
     */
    HighlightDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        var /** @type {?} */ codeElements = [];
        this.highlighter$
            .skipWhile(function () { return _this.notLoaded(); })
            .delay(this.hlDelay)
            .switchMap(function () {
            switch (_this.highlight) {
                case 'all':
                    codeElements = _this.el.querySelectorAll('pre code');
                    break;
                case '':
                    codeElements = [_this.el];
                    break;
                default:
                    codeElements = _this.el.querySelectorAll(_this.highlight);
            }
            return Observable.from(codeElements)
                .take(1)
                .map(function (code) {
                /** Highlight only If content is plain text */
                if (code.childNodes.length === 1 && code.childNodes[0].nodeName === '#text') {
                    var /** @type {?} */ highlightedCode = hljs.highlightAuto(code.innerText.trim()).value;
                    /** Render the highlighted code */
                    if (highlightedCode !== code.innerHTML) {
                        _this.renderer.setProperty(code, 'innerHTML', highlightedCode);
                    }
                }
            });
        }).subscribe();
        /** Load highlight.js script and theme */
        if (this.notLoaded()) {
            this.loadScript();
            this.loadTheme();
        }
        else {
            this.highlighter$.next();
        }
        /** Auto highlight on changes */
        if (this.hlAuto) {
            this.domObs = new MutationObserver(function () { return _this.highlighter$.next(); });
            this.domObs.observe(this.el, { childList: true, subtree: true });
        }
    };
    /**
     * @return {?}
     */
    HighlightDirective.prototype.loadScript = function () {
        var _this = this;
        var /** @type {?} */ script = this.renderer.createElement('script');
        script.async = true;
        script.type = 'text/javascript';
        script.onload = function () {
            _this.highlighter$.next();
        };
        script.src = this.hl.path + "/highlight.pack.js";
        this.renderer.setAttribute(script, 'data-timestamp', new Date().getTime().toString());
        this.renderer.appendChild(this.el, script);
    };
    /**
     * @return {?}
     */
    HighlightDirective.prototype.loadTheme = function () {
        var /** @type {?} */ style = this.renderer.createElement('link');
        style.rel = 'stylesheet';
        style.type = 'text/css';
        // style.onload = () => {
        //   console.log('theme loaded');
        // };
        style.href = this.hl.path + "/styles/" + this.hl.theme + ".css";
        this.renderer.appendChild(this.el, style);
    };
    /**
     * @return {?}
     */
    HighlightDirective.prototype.notLoaded = function () {
        return typeof hljs === 'undefined';
    };
    /**
     * @return {?}
     */
    HighlightDirective.prototype.ngOnDestroy = function () {
        if (this.hlAuto) {
            this.domObs.disconnect();
        }
        this.highlighter$.unsubscribe();
    };
    return HighlightDirective;
}());
HighlightDirective.decorators = [
    { type: Directive, args: [{
                selector: '[highlight]'
            },] },
];
/**
 * @nocollapse
 */
HighlightDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Renderer2, },
    { type: HighlightService, },
]; };
HighlightDirective.propDecorators = {
    'highlight': [{ type: Input, args: ['highlight',] },],
    'hlAuto': [{ type: Input },],
    'hlDelay': [{ type: Input },],
};
/**
 * Initialize HighlightService with theme and path
 * @param {?} theme
 * @param {?} path
 * @return {?}
 */
function HighlightFactory(theme, path) {
    return new HighlightService(theme, path);
}
var PATH = new InjectionToken('path');
var THEME = new InjectionToken('theme');
var HighlightModule = (function () {
    function HighlightModule() {
    }
    /**
     * @param {?=} theme
     * @param {?=} path
     * @return {?}
     */
    HighlightModule.forRoot = function (theme, path) {
        return {
            ngModule: HighlightModule,
            providers: [
                { provide: THEME, useValue: theme },
                { provide: PATH, useValue: path },
                {
                    provide: HighlightService,
                    useFactory: HighlightFactory,
                    deps: [THEME, PATH]
                }
            ]
        };
    };
    return HighlightModule;
}());
HighlightModule.decorators = [
    { type: NgModule, args: [{
                declarations: [HighlightDirective],
                exports: [HighlightDirective]
            },] },
];
/**
 * @nocollapse
 */
HighlightModule.ctorParameters = function () { return []; };
var HighlightUmdDirective = (function () {
    /**
     * @param {?} el
     * @param {?} renderer
     */
    function HighlightUmdDirective(el, renderer) {
        this.renderer = renderer;
        this.highlighter$ = new Subject();
        this.hlAuto = true;
        this.hlDelay = 200;
        this.el = el.nativeElement;
    }
    /**
     * @return {?}
     */
    HighlightUmdDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        var /** @type {?} */ codeElements = [];
        this.highlighter$
            .skipWhile(function () { return _this.notLoaded(); })
            .delay(this.hlDelay)
            .switchMap(function () {
            switch (_this.highlight) {
                case 'all':
                    codeElements = _this.el.querySelectorAll('pre code');
                    break;
                case '':
                    codeElements = [_this.el];
                    break;
                default:
                    codeElements = _this.el.querySelectorAll(_this.highlight);
            }
            return Observable.from(codeElements)
                .take(1)
                .map(function (code) {
                /** Highlight only If content is plain text */
                if (code.childNodes.length === 1 && code.childNodes[0].nodeName === '#text') {
                    var /** @type {?} */ highlightedCode = hljs.highlightAuto(code.innerText.trim()).value;
                    /** Render the highlighted code */
                    if (highlightedCode !== code.innerHTML) {
                        _this.renderer.setProperty(code, 'innerHTML', highlightedCode);
                    }
                }
            });
        }).subscribe();
        this.highlighter$.next();
        /** Auto highlight on changes */
        if (this.hlAuto) {
            this.domObs = new MutationObserver(function () { return _this.highlighter$.next(); });
            this.domObs.observe(this.el, { childList: true, subtree: true });
        }
    };
    /**
     * @return {?}
     */
    HighlightUmdDirective.prototype.notLoaded = function () {
        return typeof hljs === 'undefined';
    };
    /**
     * @return {?}
     */
    HighlightUmdDirective.prototype.ngOnDestroy = function () {
        if (this.hlAuto) {
            this.domObs.disconnect();
        }
        this.highlighter$.unsubscribe();
    };
    return HighlightUmdDirective;
}());
HighlightUmdDirective.decorators = [
    { type: Directive, args: [{
                selector: '[highlight]'
            },] },
];
/**
 * @nocollapse
 */
HighlightUmdDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Renderer2, },
]; };
HighlightUmdDirective.propDecorators = {
    'highlight': [{ type: Input, args: ['highlight',] },],
    'hlAuto': [{ type: Input },],
    'hlDelay': [{ type: Input },],
};
var HighlightUmdModule = (function () {
    function HighlightUmdModule() {
    }
    return HighlightUmdModule;
}());
HighlightUmdModule.decorators = [
    { type: NgModule, args: [{
                declarations: [HighlightUmdDirective],
                exports: [HighlightUmdDirective]
            },] },
];
/**
 * @nocollapse
 */
HighlightUmdModule.ctorParameters = function () { return []; };
/**
 * Generated bundle index. Do not edit.
 */
export { HighlightModule, HighlightUmdModule, HighlightUmdDirective as ɵf, HighlightDirective as ɵd, HighlightFactory as ɵa, PATH as ɵb, THEME as ɵc, HighlightService as ɵe };
//# sourceMappingURL=ngx-highlightjs.es5.js.map
